@@ -532,8 +532,8 @@ int sc_get_config_addr_info2(int token, uint32_t config_addr,
 
 	do {
 		rc = sc_rtas_call(token, 4, 2, htobe32(config_addr),
-				  BITS32_HI(htobe64(phb_id)),
-				  BITS32_LO(htobe64(phb_id)),
+				  htobe32(BITS32_HI(phb_id)),
+				  htobe32(BITS32_LO(phb_id)),
 				  htobe32(func), &status, &be_info);
 		if (rc)
 			break;
@@ -930,6 +930,7 @@ sc_platform_dump(int token, uint64_t dump_tag, uint64_t sequence, void *buffer,
 	uint32_t kernbuf_pa = 0;
 	uint32_t next_hi, next_lo;
 	uint32_t bytes_hi, bytes_lo;
+	uint32_t dump_tag_hi, dump_tag_lo;
 	void *kernbuf;
 	int status;
 	int rc;
@@ -939,22 +940,29 @@ sc_platform_dump(int token, uint64_t dump_tag, uint64_t sequence, void *buffer,
 		if (rc)
 			return rc;
 	}
+	/*
+	 * Converting a 64bit host value to 32bit BE, _hi and _lo
+	 * pair is tricky: we should convert the _hi and _lo 32bits
+	 * of the 64bit host value.
+	 */
+	dump_tag_hi = htobe32(BITS32_HI(dump_tag));
+	dump_tag_lo = htobe32(BITS32_LO(dump_tag));
+
+	next_hi = htobe32(BITS32_HI(sequence));
+	next_lo = htobe32(BITS32_LO(sequence));
 
-	*seq_next = htobe64(sequence);
 	do {
-		sequence = *seq_next;
-		rc = sc_rtas_call(token, 6, 5, BITS32_HI(htobe64(dump_tag)),
-				  BITS32_LO(htobe64(dump_tag)),
-				  BITS32_HI(sequence),
-				  BITS32_LO(sequence),
+		rc = sc_rtas_call(token, 6, 5, dump_tag_hi,
+				  dump_tag_lo,
+				  next_hi,
+				  next_lo,
 				  htobe32(kernbuf_pa), htobe32(length),
 				  &status, &next_hi, &next_lo, &bytes_hi,
 				  &bytes_lo);
 		if (rc < 0)
 			break;
-		
-		*seq_next = BITS64(next_hi, next_lo);
-		dbg1("%s: seq_next = 0x%llx\n", __FUNCTION__, *seq_next);
+		sequence = BITS64(be32toh(next_hi), be32toh(next_lo));
+		dbg1("%s: seq_next = 0x%llx\n", __FUNCTION__, sequence);
 
 		rc = handle_delay(status, &elapsed);
 	} while (rc == CALL_AGAIN);
@@ -965,9 +973,10 @@ sc_platform_dump(int token, uint64_t dump_tag, uint64_t sequence, void *buffer,
 	if (kernbuf)
 		(void)sc_free_rmo_buffer(kernbuf, kernbuf_pa, length);
 
+	*seq_next = sequence;
+	bytes_hi = be32toh(bytes_hi);
+	bytes_lo = be32toh(bytes_lo);
 	*bytes_ret = BITS64(bytes_hi, bytes_lo);
-	*seq_next = be64toh(*seq_next);
-	*bytes_ret = be64toh(*bytes_ret);
 
 	dbg1("(0x%llx, 0x%llx, %p, %d, %p, %p) = %d, 0x%llx, 0x%llx\n",
 	     dump_tag, sequence, buffer, length, seq_next, bytes_ret,
@@ -996,8 +1005,8 @@ sc_read_slot_reset(int token, uint32_t cfg_addr, uint64_t phbid, int *state,
 
 	do {
 		rc = sc_rtas_call(token, 3, 3, htobe32(cfg_addr),
-				  BITS32_HI(htobe64(phbid)),
-				  BITS32_LO(htobe64(phbid)), &status,
+				  htobe32(BITS32_HI(phbid)),
+				  htobe32(BITS32_LO(phbid)), &status,
 				  state, eeh);
 		if (rc)
 			return rc;
@@ -1114,8 +1123,8 @@ sc_set_eeh_option(int token, uint32_t cfg_addr, uint64_t phbid, int function)
 
 	do {
 		rc = sc_rtas_call(token, 4, 1, htobe32(cfg_addr),
-				  BITS32_HI(htobe64(phbid)),
-				  BITS32_LO(htobe64(phbid)),
+				  htobe32(BITS32_HI(phbid)),
+				  htobe32(BITS32_LO(phbid)),
 				  htobe32(function), &status);
 		if (rc)
 			return rc;
@@ -1312,8 +1321,8 @@ int sc_suspend_me(int token, uint64_t streamid)
 	int rc;
 
 	do {
-		rc = sc_rtas_call(token, 2, 1, BITS32_HI(htobe64(streamid)),
-				  BITS32_LO(htobe64(streamid)), &status);
+		rc = sc_rtas_call(token, 2, 1, htobe32(BITS32_HI(streamid)),
+				  htobe32(BITS32_LO(streamid)), &status);
 		if (rc)
 			return rc;
 
